#include "Python.h"


extern PyObject *
GetTopLevelWindowList(PyObject *self, PyObject *args);

extern PyObject *
GetWindowChildsList(PyObject *self, PyObject *args);

extern PyObject *
GetProcessName(PyObject *self, PyObject *args);

extern PyObject *
GetProcessDict(PyObject *self, PyObject *args);

extern PyObject *
PyWin_GetClassName(PyObject *self, PyObject *args);

extern PyObject *
PyWin_GetWindowText(PyObject *self, PyObject *args);

